# -*- coding: utf-8 -*-
"""
Created on Mon Nov 27 22:41:43 2017

@author: apoung
"""

import numpy as np
import matplotlib.pyplot as plt
import scipy.linalg as sl
from gradient_optimal import *

from mpl_toolkits.mplot3d  import Axes3D
import time as time

def f(x):
    return 1.0 + 0.0 * x
def g(x):
    return 0.5 * x * (1.0 - x)
    

def J(u, a, b, x, fx):
    v = np.array([a]+u.tolist()+[b])
    h = x[1:] - x[0:-1]
    w = (v[1:] - v[0:-1])**2
    fv = fx * v
    r = np.dot(0.5/h,w) - np.dot(h/2.0, fv[1:] + fv[0:-1])
    return r

def DJ(u, a, b, x, fx):
    n = len(u)
    v = np.array([a]+u.tolist()+[b])
    h = x[1:] - x[0:-1]
    r = np.zeros(n)
    for i in range(1,n+1):
        ri = (v[i]- v[i-1])/h[i-1] + (v[i] - v[i+1])/h[i]
        ri -= fx[i] * (h[i-1] + h[i])/2.0
        r[i-1] = ri
    return r

  
   


def question9():
    pass


def question10(fignum,rho):
    print("\n\n DEBUT ======== QUESTION   7 , rho = %3f\n\n".format(rho));
    from ScriptPartie1 import question2
    ###########################################################################    
    #  QUESTION 7-1
    ###########################################################################
    a,  b, N = g(0), g(1), 2
    A, F, x, fx = question2(a, b, N, f)
    
    xd = np.linspace (-10, 10, 40)
    X, Y = np.meshgrid(xd,xd)
    Z = X*(0.5 * A[0,0] * X + A[0,1] * Y- F[0]) + Y*(0.5 * A[1,0] * X + A[1,1] * Y- F[1])

    #    fig = 
    plt.figure(fignum)
    plt.hold(True)
    
    #ax = fig.gca()
    #ax.plot_surface(X,Y,Z)
    
    #matplotlib.pyplot.contourf(X,Y,Z,20)
    u1 = A[0,0] * X + A[0,1] * Y - F[0]
    u2 = A[1,0] * X + A[1,1] * Y - F[1]
    matplotlib.pyplot.quiver(X,Y,u1,u2)

    ###########################################################################
    # QUESTION 7-2
    ###########################################################################
    
    u0 = np.array([8, 4]);
    epsilon = 1e-12;
    
    iterMax = 10000
    stockage = 1

    Jf = lambda u : J(u, a, b, x, fx)
    DJf = lambda u : DJ(u, a, b, x, fx)
    
    u, iters,_ ,_  = gradient_optimal(Jf,DJf,u0,rho,epsilon,iterMax,stockage);

    n = iters
    
    zciso = np.array([Jf(u[i]) for i in range(n)], dtype = float)
    zciso, I = np.unique(zciso, return_index=True)
    
    u1 = np.array([u[i][0]  for i in I])
    u2 = np.array([u[i][1]  for i in I])
    
    
    matplotlib.pyplot.contour(X,Y, Z, zciso)
    
    residu = np.linalg.norm(u[-1] - u[-2]);
    if(iters == iterMax  or residu > epsilon or np.isnan(residu) ):
        print("DIVERGENCE :  residu = %e,  #iteration = %d , u1 = %e, u2 = %e\n"% (residu, iters, u1[-1], u2[-1]))
    else:
        matplotlib.pyplot.quiver(u1[0:-1],u2[0:-1], u1[1:] - u1[0:-1], u2[1:] - u2[0:-1], color='m', scale=4., scale_units='xy');
        plt.plot(u1,u2,'m')
        print(' CONVERGENCE :  residu = %e,  #iteration = %d , u1 = %e, u2 = %e\n'%(residu, iters, u1[-1], u2[-1]));
    fig = plt.figure(3)
    ax = fig.gca(projection='3d')
    ax.plot_surface(X,Y,Z,alpha=0.3)
    ax.contour(X,Y,Z,zdir='x',offset=-10,cmap = cm.coolwarm)
    #plt.plot(zciso)
    print(zciso)

def question11(fignum):
    print('\n\n DEBUT QUESTION   8 \n\n')
    plt.figure(fignum)
    plt.clf() 
    plt.hold(False)

    N = [2,5,20,50]
    colors =['r','g','b','m','k']
    leg = ['N = 2 ', 'N = 5 ',  'N = 20','N = 50','exacte']

    
    a,  b = g(0.), g(1.)
    # solution exate
    xe = np.linspace(0.,1.,100)
    ue = g(xe);
    plt.hold(True) 
    epsilon = 1e-12

    rho = 0.1
    
    iterMax = 100000
    stockage = False
    
    for k in range(len(N)):
        n = N[k]
        t0 = time.clock()
        u0 = np.zeros(n, dtype = 'float')
        x = np.linspace(0.,1.,n+2)
        fx = f(x)
        Jf = lambda u : J(u, a, b, x, fx)
        DJf = lambda u : DJ(u, a, b, x, fx)
        rho = alphaOp(DJf,n)
        un, itern,_ ,_ = gradient_optimal(Jf,DJf,u0,rho,epsilon,iterMax,stockage)
        elaps = time.clock() - t0
        
        print("pas depart %f;  ajuste a %f "%(0.1,rho))
        print('Pour N = %5d , #iterations = %5d , temps exec = %7.6f \n' %(n,itern,elaps))
        
        plt.plot(x,[a]+un.tolist()+[b], colors[k])
        
    plt.plot(xe,ue,colors[-1])
    plt.legend(leg, loc='best');
    plt.hold(False)
    print('\n\n FIN ====== QUESTION   8 \n\n')

 
def exactSol(x):
    return 0.5 * x * (1 - x)


def alphaOp(DJ,n):
    a = np.eye(n)
    b = DJ(np.zeros(n))
    for i in range(n):
        a[:,i] = DJ(a[:,i]) - b
    l = np.linalg.eig(a)[0]
    return 2.0/(np.min(l) + np.max(l))

def question11_3(figurenum):
    a,  b, N = g(0.), g(1.), 30
    xe = np.linspace(0.,1.,N+2)
    ue = g(xe)
    
    epsilon = 1e-11
    rho = 0.2
    iterMax = 100000
    stockage = False
    n = N
    t0 = time.clock()
    u0 = np.zeros(n, dtype = 'float')
    x = np.linspace(0.,1.,n+2)
    fx = f(x)
    Jf = lambda u : J(u, a, b, x, fx)
    DJf = lambda u : DJ(u, a, b, x, fx)
    un, itern, rhoL, wL = gradient_optimal(Jf,DJf,u0,rho,epsilon,iterMax,stockage)
    elaps = time.clock() - t0
    rhoL = rhoL[0::100]
    wL = wL[0::100]
    print('Pour N = %5d , #iterations = %5d , temps exec = %7.6f \n' %(n,itern,elaps))
    plt.figure(fignum)
    plt.clf()
    #plt.subplot(1,3, 1)
    plt.plot(x,[a]+un.tolist()+[b], '+',xe,ue,'-')
    plt.legend(['approx','exact'])
    plt.title('comparaison solution approchee')
   
    plt.figure()
    plt.clf()
    #plt.subplot(1,3, 2)
    rhoOp = alphaOp(DJf, n)
    plt.plot(rhoL,'-b', np.ones(len(rhoL)) * rhoOp, 'r')
    plt.legend(['alphak','alphaOp'])
    plt.title('comparaison pas optimal et pas calcules')

    plt.figure()
    plt.clf()
    #  plt.subplot(1,3, 3)
    WT = np.array(wL)
    M = np.dot(WT, WT.T)
    plt.plot(np.arange(np.size(M,0)),M)
    plt.title('courbes des colones de WT * W ')
    
    
    
    
   
if __name__=="__main__":
    fignum = 1
    rho = 0.1
    question10(fignum,rho)
    #question11(fignum+1)
    #question11_3(fignum+2)
